
import math 
from dolfin import *
from dolfin.ufl import *
dolfin_set("form compiler", "sfc")



class BC(Function): 
    def eval(self, values, xx): 
        r = math.sqrt((xx[0]-0.5)**2 + (xx[1]-0.5)**2) 

        values[0] = (xx[0]-0.5)/r 
        values[1] = (xx[1]-0.5)/r 

class DirichletBoundary(SubDomain): 
    def inside(self, x, on_boundary): 
        return bool(on_boundary)

# Geometry
N = 11  
if N%2==0: 
    print "The solution will have a singularity in a nodal point."
    #sys.exit(0)

mesh = UnitSquare(N,N)

# Function spaces
X = VectorFunctionSpace(mesh, 'CG', 1)
Y = FunctionSpace(mesh, 'CG', 1)
XY = X + Y

# Solution function (in mixed space)
u = Function(XY)

# Basis functions (in mixed space)
vv = TestFunction(XY)
uu = TrialFunction(XY)

# Form coefficients
do_split = True
if do_split:
    x, lmbda = split(u)
else:
    x = Function(X)
    lmbda = Function(Y)

# Forms
L = inner(grad(x),grad(x))*dx + dot(x,x)*lmbda*dx
if do_split:
    F = derivative(L, u, vv)
    J = derivative(F, u, uu)
else:
    F = derivative(L, (x,lmbda), vv)
    J = derivative(F, (x,lmbda), uu)

# start vector 
dirichlet_function = BC(X)
x0 = project(dirichlet_function,X)

lmbda0 = Function(Y)
lmbda0.vector().zero()

# Solve nonlinear variational problem
dirichlet_boundary = DirichletBoundary()
#plot(dirichlet_function)
bc = DirichletBC(X, dirichlet_function, dirichlet_boundary)
#problem = VariationalProblem(J, F, bc, nonlinear=True)

b = assemble(F)
A = assemble(J)
file = File("A.m"); file << A;  
file = File("b.m"); file << b;  
print "norm of b ", b.norm()
problem = VariationalProblem(J, F, bc, nonlinear=True)
problem.solve(u)

# Plot both fields TODO: Having problems with showing both
#x0.assign(u.sub(0))
#lmbda0.assign(u.sub(1))
figure(0)
plot(x0, title="x")
#plot(u.sub(0), title="x")
figure(1)
plot(lmbda0, title="lmbda")
#plot(u.sub(1), title="lmbda")
interactive()

# Write solution to file
vtk = File("x.pvd")
vtk << x0 
vtk = File("lmbda.pvd")
vtk << lmbda0

