#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This module contains tools for consistent naming of classes.
"""

__author__ = "Martin Sandve Alnes"
__date__   = "2008-08-12 -- 2009-03-05"

import re
import hashlib

def base_element_classname(element):
    #return "%s_%s_%d" % (element.family, element.cell, element.degree)
    s = repr(element).strip()
    s = re.sub("[^\w]", "_", s)
    s = re.sub("_+", "_", s)
    s = s.strip("_")
    return s

def finite_element_classname(element):
    return "fe_%s" % base_element_classname(element)

def dof_map_classname(element):
    return "dm_%s" % base_element_classname(element)

def integral_classname(integral, form_name):
    m = integral.measure()
    return "%s_%s_integral_%s" % (form_name,
        m.domain_type(), m.domain_id())

def form_classname(form, options):
    if options.code.form.name:
        form_name = options.code.form.name
    else:
        checksum = hashlib.md5(repr(form) + repr(options))
        form_name = "sfcform_%s" % checksum.hexdigest()
    return form_name
